 package simulator.workload.generator.impl;

import gridsim.ResourceCalendar;
import schedframe.ResourceContainer;
import schedframe.implementation.ResourceContainerImpl;
import org.qcg.broker.schemas.exception.NoSuchParamException;
import org.qcg.broker.schemas.exception.UnknownParameter;
import org.qcg.broker.schemas.hostparams.ComputingResource;
import org.qcg.broker.schemas.hostparams.HostParameters;
import org.qcg.broker.schemas.hostparams.wrapper.ComputingResourceParameters;
import org.qcg.broker.schemas.hostparams.wrapper.HostParametersWrapper;
import org.qcg.broker.schemas.hostparams.wrapper.QueueResourceParamExplorer;
import org.qcg.broker.schemas.hostparams.wrapper.QueuingSystem;
import org.qcg.broker.schemas.hostparams.wrapper.impl.ComputingResourceParamExplorerImpl;
import org.qcg.broker.schemas.hostparams.wrapper.impl.HostParametersWrapperImpl;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import milus.Constants;
import milus.MiloARPolicy;
import milus.MiloAllocPolicy;
import milus.MiloGridResource;
import milus.MiloPE;
import milus.MiloResourceCharacteristics;
import milus.implementation.MiloARResReqManagerImpl;

import simulator.GssARResource;
import simulator.GssConstants;
import simulator.GssResource;
import simulator.RandomNumbers;
//import simulator.factory.QueueingSystemFactory;
//import simulator.factory.ResourcesRandomNumbersFactory;
//import simulator.factory.SingleResourceFactory;
import simulator.plugin.localallocationpolicies.implementation.LocalAllocARPolicyWraper;
import simulator.plugin.localallocationpolicies.implementation.LocalAllocPolicyWraper;
import simulator.stats.GSSAccumulator;
import simulator.utils.GSSimXML;
import simulator.workload.exceptons.XMLLogicException;
import simulator.workload.generator.ResourceGeneratorInterface;

/**
 * 
 * @author Stanislaw Szczepanowski
 * @author Marcin Krystek
 *
 */

public class ResourceGenerator implements ResourceGeneratorInterface {

	/**
	 * The descriptions of the resources 
	 */
	protected ResourceContainer<ComputingResource, QueuingSystem>[] resourcesDescriptions;
	
	/**
	 * All the generated resources
	 */
	protected MiloGridResource[] miloGridResources;
	
	/**
	 * All the local allocation policies for generated resources
	 */
	protected MiloAllocPolicy[] miloAllocPolicies;
	
	/**
	 * All the resource characteristics for generated resources
	 */
	protected MiloResourceCharacteristics[] miloResourceCharacteristics;
	
	/**
	 * The path of the forecast finish time plugin
	 */
	protected Map <String, String>forecastFinishTimePlugin; 
	protected String defaultForecastFinishTimePlugin;
	
	/**
	 * The path of the local allocation policy plugin
	 */
	protected Map <String, String>localAllocationPolicyPlugin;
	protected String defaultLocalAllocationPolicyPlugin;
	
	/**
	 * Stores the xml description of all resources
	 */
	protected String xmlResourcesDescription;
	
	/**
	 * Stores the mean total load of all resources
	 */
	protected GSSAccumulator meanTotalLoad;
	
	/**
	 * Stores the mean length of queues of all resources
	 */
	protected GSSAccumulator meanQueueLength;

	/**
	 * Internal random number generator
	 */
	protected RandomNumbers randNumbers;
	
	protected File outputDir;
	
	
	/**
	 * Temporary constructor. 
	 * For test purpose only.
	 * @param forecastFinishTimePlugin  Path of the forecast finish time plugin
	 * @param localAllocationPolicyPlugin  Path of the local allocation policy plugin
	 */
	public ResourceGenerator(String forecastFinishTimePlugin, String localAllocationPolicyPlugin){
		this.defaultForecastFinishTimePlugin = forecastFinishTimePlugin;
		this.defaultLocalAllocationPolicyPlugin = localAllocationPolicyPlugin;
		this.forecastFinishTimePlugin = new HashMap<String, String>();
		this.localAllocationPolicyPlugin = new HashMap<String, String>();
		this.outputDir = null;
	}
	
	public ResourceGenerator(){
		this.defaultForecastFinishTimePlugin = "no_defaultForecastPlugin";
		this.defaultLocalAllocationPolicyPlugin = "no_defaultAllocationPlugin";
		this.forecastFinishTimePlugin = new HashMap<String, String>();
		this.localAllocationPolicyPlugin = new HashMap<String, String>();
		this.outputDir = null;
	}

	
	public void readConfigurationHostsDescriptionXMLFile(String fileName) throws FileNotFoundException {
		FileReader reader = new FileReader(fileName);
		HostParameters hostParameters = GSSimXML.unmarshal(HostParameters.class, reader);
		xmlResourcesDescription = hostParameters.toString();
		setUpFields(hostParameters);
	}
	
	public void readConfigurationXMLFile(String fileName) throws FileNotFoundException, XMLLogicException, IllegalAccessException {
		throw new RuntimeException("Not implemented.");
	/*	
		FileReader reader = new FileReader(fileName);
		ResourcesConfiguration resConfig = GSSimXML.unmarshal(ResourcesConfiguration.class, reader);
		ResourceConfigValidator resConfigvalidator = new ResourceConfigValidator(resConfig);
		
		if(!resConfigvalidator.validate())
			return;
		
		ResourcesRandomNumbersFactory randNumFactory = new ResourcesRandomNumbersFactory();
		randNumbers = randNumFactory.initRandomNumbers(resConfig);
		
		simulator.hostspec.SingleResource singleRes = resConfig.getSingleResource();
		simulator.hostspec.QueueingSystem queueSystem = resConfig.getQueueingSystem();
		
		HostParameters hostParameters = new HostParameters();
		
		if(singleRes != null){
			SingleResourceFactory resFactory = new SingleResourceFactory(singleRes, randNumbers);
			hostParameters = resFactory.createResources(hostParameters);
			forecastFinishTimePlugin.putAll(resFactory.getForecastFinishTimePlugin());
			localAllocationPolicyPlugin.putAll(resFactory.getLocalAllocationPolicyPlugin());
		}
		
		if(queueSystem != null) {
			QueueingSystemFactory queueSysFactory = new QueueingSystemFactory(queueSystem, randNumbers);
			hostParameters = queueSysFactory.createQueueingSystem(hostParameters);
			forecastFinishTimePlugin.putAll(queueSysFactory.getForecastFinishTimePlugin());
			localAllocationPolicyPlugin.putAll(queueSysFactory.getLocalAllocationPolicyPlugin());
		}
			
		xmlResourcesDescription = hostParameters.toString();
		setUpFields(hostParameters);
		
		if(outputDir != null)
			writeConfiguration(hostParameters);
			*/
	}
	
	public MiloGridResource[] getCreatedResources() {
		return miloGridResources;
	}
	
	/**
	 * Sets all the necessary data after reading the configuration
	 * @param hostParam
	 */
	protected void setUpFields(HostParameters hostParam) {
		int count = 0;
		HostParametersWrapper hpWrapper = new HostParametersWrapperImpl();
		hpWrapper.wrap(hostParam);
		ResourceContainer<ComputingResource, QueuingSystem> hpItem = null;
		
		ComputingResource compRes[] = null;
		try {
			compRes = hpWrapper.getComputingResources();
		} catch (UnknownParameter e) {
			e.printStackTrace();
		}
		count = compRes.length;
		resourcesDescriptions = new ResourceContainerImpl[count];
		for(int i = 0; i < compRes.length; i++){
			resourcesDescriptions[i] = new ResourceContainerImpl(compRes[i]);
		}
		
		try {
			if(hpWrapper.getQueueResources() != null){
				if(log.isWarnEnabled())
					log.warn("Queues are currently not supported.");
			}
		} catch (UnknownParameter e) {
			e.printStackTrace();
		}
		miloGridResources = new MiloGridResource[count];
		miloAllocPolicies = new MiloAllocPolicy[count];
		miloResourceCharacteristics = new MiloResourceCharacteristics[count];
	}
	
	public void createResources(ResourceCalendar resCalendar) throws Exception {
		boolean reservationAvailable = false;
		String forecastPlugin = "";
		String allocationPlugin = "";
		String name;
		
		ComputingResourceParameters crp = new ComputingResourceParamExplorerImpl();
		
		for (int i = 0; i < resourcesDescriptions.length; ++i) {
			QueuingSystem queueingSystem = resourcesDescriptions[i].getQueueingSystem();
			ComputingResource singleResource = resourcesDescriptions[i].getSingleResource();
			
			
			Map<String, Object> resDesc = null;
			String resManContact = null;
			if ( queueingSystem != null) {
				
		/*		name = queueingSystem.getFrontendId();
				if(forecastFinishTimePlugin.containsKey(name))
					forecastPlugin = forecastFinishTimePlugin.get(name);
				else
					forecastPlugin = defaultForecastFinishTimePlugin;
				if(localAllocationPolicyPlugin.containsKey(name))
					allocationPlugin = localAllocationPolicyPlugin.get(name);
				else
					allocationPlugin = defaultLocalAllocationPolicyPlugin;
				
				reservationAvailable = true;
				resManContact = queueingSystem.getFrontendId();
				
				
				QueuesGroup[] queuesGroups = queueingSystem.getQueuesGroup();
				for (int k = 0; k < queuesGroups.length; ++k) {
					//assumption queuesGroups.length == 1
					
					QueuesGroup queuesGroup = queuesGroups[k];
					for (int q = 0; q < queuesGroup.getQueueCount(); ++q) {
						QueueResourcesCollective qrc = queuesGroup.getQueuesGroupChoice().getQueueResourcesCollective();
						if (qrc != null) {
							ClusterMachines clusterMachines = qrc.getClusterMachines();
							resDesc = getClusterMachinesCharacteristics(clusterMachines);
						} else {
							QueueResource[] queueResource = queuesGroup.getQueuesGroupChoice().getQueueResource();

							for (int qr = 0; qr < queueResource.length; ++i) {
								QueueResource qRes = queueResource[qr];
								Machine machine = qRes.getMachine();
								resDesc = getMachineCharacteristics(machine);
								break;
							}
						}
						
						if (resDesc != null) {
							break;
						}
					}
					if (resDesc != null) {
						break;
					}
				}
				*/
			} else if (singleResource != null) {
				
				name = singleResource.getResourceId();
				if(forecastFinishTimePlugin.containsKey(name))
					forecastPlugin = forecastFinishTimePlugin.get(name);
				else
					forecastPlugin = defaultForecastFinishTimePlugin;
				if(localAllocationPolicyPlugin.containsKey(name))
					allocationPlugin = localAllocationPolicyPlugin.get(name);
				else
					allocationPlugin = defaultLocalAllocationPolicyPlugin;
				
				reservationAvailable = true;
				resManContact = singleResource.getResourceId();
				crp.explore(singleResource);
				resDesc = getMachineCharacteristics(crp);
			}	
			
			double timeZone = 0.0; // time zone this resource located

			MiloResourceCharacteristics resConfig = new MiloResourceCharacteristics(resDesc, timeZone, new MiloARResReqManagerImpl(resDesc));

			// NOTE: allocation policy in here is set to
			// ResourceCharacteristics.ADVANCE_RESERVATION not SPACE_SHARED nor TIME_SHARED.
			/*ResourceCharacteristics resConfig = new ResourceCharacteristics(arch,
				 os, mList, ResourceCharacteristics.TIME_SHARED, time_zone, cost);*/

			// 7. Finally, we need to create a GridResource object.
	
			MiloGridResource gridRes = null;

			// use this AR scheduling algorithm. The name of this entity
			// will be name_scheduler, e.g. Resource0_scheduler.
			
			MiloAllocPolicy policy;
			//ARSimpleSpaceShared policy = new ARSimpleSpaceShared(name,scheduler);
			
			if(reservationAvailable){
				// NOTE: You need to use a grid resource entity that supports
				// advanced reservation functionalities.
				policy = new LocalAllocARPolicyWraper(resManContact, GssConstants.DEFAULT_RESOURCE_MANAGER_NAME, allocationPlugin, forecastPlugin, resourcesDescriptions[i]);
				gridRes = new GssARResource(resManContact, resConfig, resCalendar, (MiloARPolicy) policy, resourcesDescriptions[i]);
				
			} else {
				policy = new LocalAllocPolicyWraper(resManContact, GssConstants.DEFAULT_RESOURCE_MANAGER_NAME, allocationPlugin, forecastPlugin, resourcesDescriptions[i]);
				gridRes = new GssResource(resManContact, resConfig, resCalendar, policy, resourcesDescriptions[i]);
			}

			miloGridResources[i] = gridRes;
			miloAllocPolicies[i] = policy;
			miloResourceCharacteristics[i] = resConfig;
		}
	}
	
	/**
	 * Creates a resource description map on the basis of given machine
	 * @param machine the machine that is to be read
	 * @return map of resource characteristics
	 */
	protected Map<String, Object> getMachineCharacteristics(ComputingResourceParameters machine) {
		//Here are the steps needed to create a Grid resource:
		// 1. We need to create an object of MachineList to store one or more
		// Machines
		//MachineList mList = new MachineList();
		
		//cpucount
		double values[] = null;
		try {
			values = machine.getCpucount();
		} catch (NoSuchParamException e) {
			e.printStackTrace();
		}
		int cpucount = Double.valueOf(values[0]).intValue();
		
		// 2. A Machine contains one or more PEs or CPUs. Therefore, should
		// create an object of PEList to store these PEs before creating
		// a Machine.
		List<MiloPE> peList = new ArrayList<MiloPE>(cpucount);
		
		//cpuspeed
		int cpuspeed = 1;
		try {
			values = machine.getCpuspeed();
			cpuspeed = Double.valueOf(values[0]).intValue();
		} catch (NoSuchParamException e) {
			if(log.isWarnEnabled())
				log.warn("Resource cpu speed parameter is not defined, assuming that cpuspeed = 1");
		}
		
		
		// 3. Create PEs and add these into an object of PEList.
		for (int cpuNumber = 0; cpuNumber < cpucount; ++cpuNumber) {				
			peList.add(new MiloPE(cpuNumber, cpuspeed)); // need to store PE id and MIPS Rating
		}	
		
		// 4. Create one Machine with its id and list of PEs or CPUs
		//mList.add(new Machine(0, peList)); // First Machine
		
		// 5. Repeat the process from 2 if we want to create more Machines
		// NOTE: if you only want to create one Machine for one Grid resource,
		// then you could skip this step.
		
		//...
		
		// 6. Create a ResourceCharacteristics object that stores the
		// properties of a Grid resource: architecture, OS, list of
		// Machines, allocation policy: time- or space-shared, time zone
		// and its price (G$/PE time unit).
		
		/* Data from the xml file **/
		//osname
		//String osName = machine.getOsName();
		
		//ostype
		//String osType = machine.getOsType();
		
		//osversion
		//String osVersion = machine.getOsVersion();
		
		//osrelease
		//String osRelease = machine.getOsRelease();
		
		//memory
		int totalMemory = 0;
		try {
			values = machine.getMemory();
			totalMemory = Double.valueOf(values[0]).intValue();
		} catch (NoSuchParamException e) {
			if(log.isWarnEnabled())
				log.warn("Resource memory parameter is not defined, assuming that totalMemory = 0");
		}
		
		
		//bandwidth
		//Bandwidth bandwidth = resource.getBandwidth();			
		//hostname
		
		//latency
		//Latency latency = resource.getLatency();
		//hostname
		
		//capacity
		//Capacity capacity = resource.getCapacity();
		//hostname
		
		//applications
		//Applications applications = resource.getApplications();
		
		//freememory
		//MemoryFree freeMemory = machine.getMemoryFree();
		
		//diskspace
		//int totalDiskSpace = machine.getDiskTotal();
		
		//freediskspace
		//DiskFree freeDiskSpace = machine.getDiskFree();
		
		//queue
		//String queue = resource.getQueue();
		
		//freecpus
		//int freecpus = resource.getFreecpus();
		
		double cost = GssConstants.DEFAULT_RESOURCE_PROCESSING_COST;//FIXME // the cost of using this resource
		
		Map<String, Object> resDesc = new HashMap<String, Object>();
		resDesc.put(Constants.MEMORY, new Integer(totalMemory));
		resDesc.put(Constants.PE, peList);
		resDesc.put(Constants.COST_PER_SEC, new Double(cost));
		return resDesc;
	}

	/**
	 * Creates a resource description map on the basis of given cluster machines
	 * @param clusterMachines the cluster machines that are to be read
	 * @return map of resource characteristics
	 */
	protected Map<String, Object> getClusterMachinesCharacteristics(QueueResourceParamExplorer clusterMachines) {
		//Here are the steps needed to create a Grid resource:
		// 1. We need to create an object of MachineList to store one or more
		// Machines
		//MachineList mList = new MachineList();
/*
		//cpucount
		double values[] = null;
		
		int cpucount = clusterMachines.getTotalCpus();
		clusterMachines.get
		// 2. A Machine contains one or more PEs or CPUs. Therefore, should
		// create an object of PEList to store these PEs before creating
		// a Machine.
		List<MiloPE> peList = new ArrayList<MiloPE>(cpucount);
		
		//cpuspeed
		int cpuspeed = clusterMachines.getNodeCpuSpeed();
		
		// 3. Create PEs and add these into an object of PEList.
		for (int cpuNumber = 0; cpuNumber < cpucount; ++cpuNumber) {				
			peList.add(new MiloPE(cpuNumber, cpuspeed)); // need to store PE id and MIPS Rating
		}	
		
		//memory
		int totalMemory = clusterMachines.getMaxFreeMemory();
		
		double cost = GssConstants.DEFAULT_RESOURCE_PROCESSING_COST;//FIXME // the cost of using this resource
		
		Map<String, Object> resDesc = new HashMap<String, Object>();
		resDesc.put(Constants.MEMORY, new Integer(totalMemory));
		resDesc.put(Constants.PE, peList);
		resDesc.put(Constants.COST_PER_SEC, new Double(cost));
		return resDesc;
		*/
		return null;
	}
	

	
	/**
	 * Reads plugins configuration from text file.
	 * File format is:
	 * 
	 * ForecastFinishTimePluginName 
	 * resource_name_1=ForecastFinishTimePluginName_ClassName_1
	 * resource_name_2=ForecastFinishTimePluginName_ClassName_2
	 * 
	 * LocalAllocationPolicyPluginName 
	 * resource_name_1=LocalAllocationPolicyPluginName_ClassName_1
	 * resource_name_2=LocalAllocationPolicyPluginName_ClassName_2
	 */
	public boolean loadPlugins(String fileName) throws IOException{
		File file = new File(fileName);
		if(!file.exists())
			throw new FileNotFoundException("No such file: "+fileName);
		
		Reader r = new FileReader(file);
		BufferedReader reader = new BufferedReader(r);
		String line = null;
		String tab[] = null;
		int s = 0;
		
		while((line = reader.readLine()) != null) {
			line = line.trim();
			
			if(line.startsWith("#")) {
				continue;
			} else if("ForecastFinishTimePluginName".equals(line)){
				s = 1;
			} else if("LocalAllocationPolicyPluginName".equals(line)){
				s = 2;
			} else {
				tab = line.split("=");
				if(tab == null || tab.length != 2) continue;
				
				switch(s) {
					case 1: forecastFinishTimePlugin.put(tab[0], tab[1]); 
						break;
					case 2: localAllocationPolicyPlugin.put(tab[0], tab[1]);
						break;
				}
			}
		}
		
		return true;
	}
	
	public void setOutputDirectory(String directoryName, boolean overwrite) throws FileNotFoundException{
		File file = new File(directoryName);

		if(file.exists()) {
			if(overwrite){
				file.delete();
				file.mkdirs();
			} else {
				throw new FileNotFoundException("Directory "+directoryName+" already exists.");
			}
		} else {
			file.mkdirs();
		}
		
		outputDir = file;

	}
	
	protected void writeConfiguration(HostParameters hostParameters){
		File file = null;
		Writer writer = null;
		try {
			
			
			file = new File(outputDir, "ResourceDescription.xml");
			writer = new FileWriter(file);
			if (! GSSimXML.marshal(hostParameters, writer))
				if(log.isErrorEnabled())
					log.error("Error in marshalling");
			
			writer.close();
			
			file = new File(outputDir, "plugins.txt");
			writer = new FileWriter(file);
			
			writer.write("# file with plugins configuration\n");
			writer.write("# file format:\n");
			writer.write("# two sections - ForecastFinishTimePluginName and LocalAllocationPolicyPluginName\n"); 
			writer.write("# each esction consists of resource name and plugin name for this particular resource\n");
			writer.write("# eg. example_resource_1=pl.psnc.simulator.forecastTimePlugin\n");
			
			writer.write("\n");
			writer.write("LocalAllocationPolicyPluginName\n");
			Set<String> set = localAllocationPolicyPlugin.keySet();
			Iterator<String> it = set.iterator();
			String key;
			while(it.hasNext()){
				key = it.next();
				writer.write(key + "=" +localAllocationPolicyPlugin.get(key)+"\n");
			}
			
			writer.write("\n");
			writer.write("ForecastFinishTimePluginName\n");
			set = forecastFinishTimePlugin.keySet();
			it =  set.iterator();
			while(it.hasNext()){
				key = it.next();
				writer.write(key + "=" +forecastFinishTimePlugin.get(key)+"\n");
			}
			writer.close();
			
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if(writer != null)
				try {
					writer.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
		}

	}
	
}